@@ -255,7 +255,14 @@ get_property (GObject *object, guint prop_id,
 static void
 entry_changed (GWeatherLocationEntry *entry)
 {
-    entry->priv->custom_text = TRUE;
+    const gchar *text;
+
+    text = gtk_entry_get_text (GTK_ENTRY (entry));
+
+    if (text && *text)
+	entry->priv->custom_text = TRUE;
+    else
+	set_location_internal (entry, NULL, NULL, NULL);
 }
 
 static void
@@ -576,8 +583,12 @@ static char *
 find_word (const char *full_name, const char *word, int word_len,
 	   gboolean whole_word, gboolean is_first_word)
 {
-    char *p = (char *)full_name - 1;
+    char *p;
 
+    if (word == NULL || *word == '\0')
+        return NULL;
+
+    p = (char *)full_name - 1;
     while ((p = strchr (p + 1, *word))) {
 	if (strncmp (p, word, word_len) != 0)
 	    continue;
